/*****************************************************************************
 * main.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2000 Chris Pinkham
 * Released under the terms of the GPL.
 * *NO WARRANTY*
 *
 * cpinkham@infi.net
 * http://www4.infi.net/~cpinkham/gyach/
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "interface.h"
#include "support.h"

#include "yahoolib.h"

#include "gyach.h"
#include "commands.h"
#include "callbacks.h"
#include "images.h"
#include "yahoochat.h"

/* 
 * Portions copyright David J. Binette from the yahmini source code
 * ALL RIGHTS RESERVED 
 */ 

/* ------------------------------------------------------------------------ */ 
/* tab stops are set to 4 */ 
/* ------------------------------------------------------------------------ */ 
 
#include "nix.h" 

 
/* ------------------------------------------------------------------------ */ 
 
SESSION session ,*sess = &session; /* for consistency */ 
SESSION * GlobalSessionHook = NULL; 
 

typedef enum
{ Medium, Bold }
FontWeight;
typedef enum
{ Italic, Roman }
FontSlant;

static GdkFont *get_font(const char *family, const short size,
	const FontWeight weight, const FontSlant slant)
{
	GdkFont *font = NULL;
	char XLFD[1000];
	XFontStruct *fontOK = (XFontStruct *) NULL;
	char my_font_name[129];
	char *ptr;
	char *from;
	char *to;

	DBG( 21, "get_font( '%s', %d, %d, %d )\n", family, size, weight, slant );

	strcpy( my_font_name, font_name );
	strcpy(XLFD, "-*");			/* foundry */

	if (family && *family)
	{
		char *p = NULL, *lowerFace = NULL;

		strcat(XLFD, "-");
		p = lowerFace = strdup(family);
		while (p && *p)
		{
			int c = tolower(*p);

			*p = c;
			++p;
		}
		strcat(XLFD, lowerFace);
		free(lowerFace);
	}
	else
	{
		strcat(XLFD, "-times");	/* default family, I should really get this from the current theme */
	}

	switch (weight)
	{
		case Bold:
			strcat(XLFD, "-bold");	/* weight */
			if (( ptr = strstr( my_font_name, "medium" ))) {
				strcpy( ptr, "bold" );
				to = ptr + 4;
				from = ptr + 6;
				while( *from ) {
					*(to++) = *(from++);
				}
				*to = '\0';
			}
			break;

		case Medium:
		default:
			strcat(XLFD, "-medium");	/* weight */
			if (( ptr = strstr( my_font_name, "bold" ))) {
				from = strstr( font_name, "bold" ) + 4;
				strcpy( ptr, "medium" );
				to = ptr + 6;
				while( *from ) {
					*(to++) = *(from++);
				}
				*to = '\0';
			}
			break;
	}

	switch (slant)
	{
		case Italic:
			strcat(XLFD, "-i");	/* slant : italic */
			break;

		case Roman:
		default:
			strcat(XLFD, "-r");	/* slant : roman */
			break;
	}

	strcat(XLFD, "-normal");	/* set width */
	strcat(XLFD, "-*");			/* additional style (nil) */

	/* dunno why, but he appears to have had these next 2 fields reversed, it */
	/* works better this way. */
	/*
	if (size > -1)
	{
		char sizeS[10];

		sprintf(sizeS, "-%d", 10 * size);
		strcat(XLFD, sizeS);
	}
	else
	{
		strcat(XLFD, "-*");
	}
	*/

	{
		char sizeS[10];

		sprintf(sizeS, "-%d", font_size);
		strcat(XLFD, sizeS);
	}
	strcat(XLFD, "-*");			/* pixel size (don't care) */

	strcat(XLFD, "-*");			/* resolution X */
	strcat(XLFD, "-*");			/* resolution Y */
	strcat(XLFD, "-*");			/* proportional spacing */
	strcat(XLFD, "-*");			/* Average width */
	strcat(XLFD, "-*");			/* Charset registry */
	strcat(XLFD, "-*");			/* Charset encoding */

	/*
	printf( "XLFD: %s\n", XLFD );
	printf( "  my: %s\n", my_font_name );
	fflush( stdout );
	*/

	if ( ! show_fonts ) {
		strcpy( XLFD, my_font_name );
	}

	/* Test load the font.  If it fails, fontOK is null, and get on with
	 * life after giving the user an error.
	 * If it is successful, immediately unload the font, since we're using
	 * GTK+ here, mmmk?
	 */
	fontOK = XLoadQueryFont(GDK_DISPLAY(), XLFD);

	if (fontOK == (XFontStruct *) NULL || (!fontOK))
	{
		/* Font was no good - you can't load this font. */
		font = NULL;
	}
	else
	{
		/* Remember to free this since XLoadQueryFont returns allocated memory */
		XFreeFont(GDK_DISPLAY(), fontOK);
		font = gdk_font_load(XLFD);
	}

	if ( ! font ) {
		strcpy( XLFD, "-*-times-medium-r-normal-*-14-*-*-*-*-*-*-*" );
		font = gdk_font_load( XLFD );
	}

	return font;
}

static GdkFont *getFont(int bold, int italic, int underline,
	const char *family, const char *sizeS)
{
	static GdkFont *font = NULL;
	FontWeight weight;
	FontSlant slant;
	short size = 0;

	DBG( 21, "getFont( %d, %d, %d, '%s', '%s' )\n",
		bold, italic, underline, family, sizeS );

	if (sizeS && *sizeS)
	{
		size = atoi(sizeS);
	}
	weight = bold ? Bold : Medium;
	slant = italic ? Italic : Roman;

	if (NULL != font)
	{
		gdk_font_unref(font);
		font = NULL;
	}

	if (NULL == (font = get_font(family, size, weight, slant)))
	{
		/* If the getting the actual font fails,
		   first try a few sizes smaller and
		   then try a blank family */
		short trySize = size - 1, endTrySize = (size - 4);

		if (family && *family)
		{
			while ((trySize > 0) &&
				(NULL == (font = get_font(family, trySize, weight, slant))) &&
				(trySize > endTrySize))
			{
				--trySize;
			}
		}

		if (NULL == font)
		{
			trySize = size;
			while ((trySize > 0) &&
				(NULL == (font = get_font(NULL, trySize, weight, slant))) &&
				(trySize > endTrySize))
			{
				--trySize;
			}
		}

	}

	if (NULL == font)
	{
		gdk_font_ref(font);
	}

	return (font);
}

/* Lookup colors based on rgb values */
GdkColor *gtkyahoo_window_alloc_color(GtkWidget * window, short red,
	short green, short blue, GdkColor * color)
{
	DBG( 21, "gtkyahoo_window_alloc_color( %p, %d, %d, %d, %p )\n",
		window, red, green, blue, color );

	/* the color we want to use */
	if (!color)
		color = (GdkColor *) malloc(sizeof(GdkColor));

	/* red, green, and blue are passed values, indicating the RGB triple
	 * of the color we want to draw. Note that the values of the RGB components
	 * within the GdkColor are taken from 0 to 65535, not 0 to 255.
	 */
	color->red = red * (65535 / 255);
	color->green = green * (65535 / 255);
	color->blue = blue * (65535 / 255);

	/* color->pixel is filled in by gtk_colormap_alloc_color(), request
	   a best match color that we will not adjust, multiple requests
	   for the same color will not eat additional entries */
	gdk_colormap_alloc_color(gtk_widget_get_colormap(window),
		color, FALSE, TRUE);

	return color;
}

static int bold = FALSE, italic = FALSE, underline = FALSE, url = FALSE;

void append_to_textbox(GtkWidget * window, GtkWidget * textbox, char *text)
{
	int i = 0;
	GdkColor colorspace, oldColorSpace;
	GdkColor *color = &colorspace;
	GdkColor *oldColor = &oldColorSpace;
	GdkFont *font = NULL;
	GtkAdjustment * adj;
	char face[1000] = "", size[1000] = "";
	int tmp_color;
	char *tmp_ptr;
	guint text_length;
	int scroll = FALSE;

	DBG( 21, "append_to_textbox( %p, %p, '%s' )\n",
		window, textbox, text );

	*face = *size = '\0';

	strcpy( face, font_family );

	font = getFont(bold, italic, underline, face, size);

	if ( force_lowercase )
		lower_str( text );

	strip_junk( text );
	convert_tags( text );

	adj = GTK_TEXT(textbox)->vadj;
	if ( adj->value == ( adj->upper - adj->lower - adj->page_size ))
		scroll = TRUE;

	text_length = gtk_text_get_length(GTK_TEXT(textbox));

	if ( buffer_size ) {
		GtkEditable *text_editable;

		gtk_text_freeze(GTK_TEXT(textbox));
		if ( text_length > ( buffer_size * 1024 )) {
			/* this code appears to break sometimes when app is */
			/* displayed remotely.  don't know why though. */
			text_editable = &GTK_TEXT(textbox)->editable;
			gtk_editable_delete_text( GTK_EDITABLE(text_editable),
				0, 512 );
			text_length = gtk_text_get_length(GTK_TEXT(textbox));
			gtk_text_set_point(GTK_TEXT(textbox), text_length );
			scroll = TRUE;
		}
		gtk_text_thaw(GTK_TEXT(textbox));
	}

	/*
	gtk_text_set_point(GTK_TEXT(textbox), text_length );
	*/

	/* Reset all attributes - currently just color */
	color = gtkyahoo_window_alloc_color(window, 0x00, 0x00, 0x00, color);

	/* freeze before printing to speed up display, especially on blended */
	/* colors or for people who switch colors every letter */
	gtk_text_freeze( GTK_TEXT(textbox));

	i = 0;
	while (text && i < strlen(text))
	{
		if (!strncmp(&text[i], YAHOO_FONT_TAG_START,
				strlen(YAHOO_FONT_TAG_START)))
		{
			int closeFound = FALSE;

			i += strlen(YAHOO_FONT_TAG_START);
			while (!closeFound)
			{
				if (!strncmp(&text[i], YAHOO_FONT_FACE_START,
						strlen(YAHOO_FONT_FACE_START)))
				{
					int k = 0;

					i += strlen(YAHOO_FONT_FACE_START);
					while (text && i < strlen(text) && text[i] != '"')
					{
						face[k] = text[i];
						++i;
						++k;
						face[k] = '\0';
					}
					while (text && i < strlen(text)
						&& NULL != strchr("\" \t", text[i]))
					{
						++i;
					}
				}

				if (text && '>' == text[i])
				{
					++i;
					closeFound = TRUE;
					continue;
				}

				if (!strncmp(&text[i], YAHOO_FONT_SIZE_START,
						strlen(YAHOO_FONT_SIZE_START)))
				{
					int k = 0;

					i += strlen(YAHOO_FONT_SIZE_START);
					while (text && i < strlen(text) && text[i] != '"')
					{
						size[k] = text[i];
						++i;
						++k;
						size[k] = '\0';
					}
					while (text && i < strlen(text)
						&& NULL != strchr("\" \t", text[i]))
					{
						++i;
					}
				}
			}

			if ( show_fonts ) /* should we use their font? */
			{
				font = getFont(bold, italic, underline, face, size);
			}
		}
		else if (( text[i] == 033 ) && ( text[i+1] == '[' ))
		{
			i += 2;  /* skip over the escape and [ */
			if ( text[i+1] == ';' ) {  /* we have a bold/not bold */
				if ( text[i] == '0' ) {
					bold = FALSE;
				} else {
					bold = TRUE;
				}
			    font = getFont(bold, italic, underline, face, size);
				i += 2;
			}
			if ( text[i] == 'x' ) {
				/* turn something OFF */
				i++;
				switch ( text[i] ) {
					case '1': bold = FALSE ;
							  font = getFont(bold, italic, underline, face,
							  		size);
					          break;
					case '2': italic = FALSE;
							  font = getFont(bold, italic, underline, face,
							  		size);
					          break;
					case '4': underline = FALSE;
							  font = getFont(bold, italic, underline, face,
							  		size);
					          break;
					case 'l': url = FALSE ;
							  memcpy(color, oldColor, sizeof(GdkColor));
							  font = getFont(bold, italic, underline, face,
							  	size);
							  break;
				}
				i += 2;
				continue;
			} else if ( text[i] == '#' ) {
				/* we found a blended color */
				unsigned int r, g, b;
				char tmp_int[3];

				if ( show_blended_colors ) {
					tmp_int[2] = '\0';

					tmp_int[0] = text[ i + 1 ];
					tmp_int[1] = text[ i + 2 ];
					sscanf( tmp_int, "%x", &r );

					tmp_int[0] = text[ i + 3 ];
					tmp_int[1] = text[ i + 4 ];
					sscanf( tmp_int, "%x", &g );

					tmp_int[0] = text[ i + 5 ];
					tmp_int[1] = text[ i + 6 ];
					sscanf( tmp_int, "%x", &b );

			    	memcpy(oldColor, color, sizeof(GdkColor));
			    	color = gtkyahoo_window_alloc_color(window, r, g, b, color);
				}

			    i += 8;
			    continue;
			} else {
				tmp_color = atoi( &text[i] );
				switch ( tmp_color ) {
					case 0: if (( text[i] == 'l' ) && ( text[i+1] == 'm' )) {
							  if ( show_colors ) {
					        	url = TRUE ;
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x00, 0x00, 0xff, color);
							  }
							  i += 2;
							  continue;
							} else if ( text[i+1] == 'm' ) {
							  italic = FALSE;
							  bold = FALSE;
							  underline = FALSE;
							  url = FALSE;
							  font = getFont(bold, italic, underline, face,
							  	size);

							  /* black */
							  memcpy(oldColor, color, sizeof(GdkColor));
							  color = gtkyahoo_window_alloc_color(window,
									0x00, 0x00, 0x00, color);
							  i += 2;
							  continue;
							}
						    break;
					case 1: bold = TRUE ;
							if ( show_fonts )
								font = getFont(bold, italic, underline, face,
									size);
							i += 2;
							continue;
					        break;
					case 2: italic = TRUE;
							if ( show_fonts )
								font = getFont(bold, italic, underline, face,
									size);
							i += 2;
							continue;
					        break;
					case 4: underline = TRUE;
							if ( show_fonts )
								font = getFont(bold, italic, underline, face,
									size);
							i += 2;
							continue;
					        break;
					case 30: /* black */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x00, 0x00, 0x00, color);
							}
							i += 3;
							break;
					case 31: /* blue */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x00, 0x00, 0xff, color);
							}
							i += 3;
						    break;
					case 32: /* cyan */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x00, 0xb2, 0xb2, color);
							}
							i += 3;
						    break;
					case 33: /* gray */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x80, 0x80, 0x80, color);
							}
							i += 3;
						    break;
					case 34: /* green */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x00, 0xc2, 0x00, color);
							}
							i += 3;
						    break;
					case 35: /* pink */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0xff, 0xaf, 0xaf, color);
							}
							i += 3;
						    break;
					case 36: /* purple */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0xb2, 0x00, 0xb2, color);
							}
							i += 3;
						    break;
					case 37: /* yellow */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0xff, 0xff, 0x00, color);
							}
							i += 3;
						    break;
					case 38: /* red */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0xff, 0x00, 0x00, color);
							}
							i += 3;
						    break;
					case 39: /* olive */
							if ( show_colors ) {
								memcpy(oldColor, color, sizeof(GdkColor));
								color = gtkyahoo_window_alloc_color(window,
									0x54, 0x6b, 0x50, color);
							}
							i += 3;
						    break;
				}
			}
			continue;
		} else if ( text[i] == 033 ) {
			i++;
		}

		if (i >= strlen(text))
		{
			break;
		}

		if ( text[i] == 033 ) {
			continue;
		}

		/* print out as much as we can */
		if (( tmp_ptr = strchr( &text[i], 033 )) != NULL ) {
			tmp_color = tmp_ptr - &text[i];
		} else {
			tmp_color = strlen( &text[i] );
		}

		gtk_text_insert(GTK_TEXT(textbox), font, color,
			NULL, (gchar *) & text[i], tmp_color );

		if ( capture_fp ) {
			fwrite( &text[i], 1, tmp_color, capture_fp );
		}

		i += tmp_color;
	}

	gtk_text_thaw( GTK_TEXT(textbox));

	if( scroll ) {
		gtk_adjustment_set_value( adj,
			adj->upper - adj->lower - adj->page_size );
	}

	if ( capture_fp )
		fflush( capture_fp );

	DBG( 22, "RETURN append_to_textbox()\n" );
}

void append_to_textbox_color(GtkWidget * win, GtkWidget * tb, char *text) {
	int old_colors = show_colors;
	int old_fonts = show_fonts;
	int old_lower = force_lowercase;

	DBG( 21, "append_to_textbox_color( %p, %p, '%s' )\n", win, tb, text );

	show_colors = 1;
	show_fonts = 1;
	force_lowercase = 0;

	/* reset things to normal */
	if ( bold )
		append_to_textbox( win, tb, YAHOO_STYLE_BOLDOFF );

	if ( italic )
		append_to_textbox( win, tb, YAHOO_STYLE_ITALICOFF );

	if ( underline )
		append_to_textbox( win, tb, YAHOO_STYLE_UNDERLINEOFF );

	if ( url )
		append_to_textbox( win, tb, YAHOO_STYLE_URLOFF );

	/* now send our text */
	append_to_textbox( win, tb, text );

	show_colors = old_colors;
	show_fonts = old_fonts;
	force_lowercase = old_lower;

	DBG( 22, "RETURN append_to_textbox_color()\n" );
}

/* ------------------------------------------------------------------------ */ 
 
int SessionHook(SESSION *sess ,short state ,short argc ,void* args) 
{ 
    int r = TRUE; 
	char buf[4096];
 
    switch(state) { 
    /* the program has started */ 
    /* user data has not yet been entered */ 
    case SHOOK_PROG: 
        GlobalSessionHook = sess; 
        break; 
 
    /* user data is complete */ 
    /* prepare to start networking */ 
    /* login follows this */ 
    case SHOOK_NET: 
        break; 
 
    /* login was successful */ 
    /* next is login to chat */ 
    case SHOOK_CHAT: 
        break; 
 
    /* connected to chat */ 
    /* ready to start receiving data */ 
    case SHOOK_RECV: 
		gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
        sprintf(buf ,"Waiting for Yahoo! greeting..."); 
		gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, buf );
		process_gtk_events();
        break; 
 
    /* yahoo data packet has arrived */ 
    case SHOOK_DATA: 
        /* on a login packet ,do a room join */ 
        if(sess->yin.type == 0x01) 
        {
			gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
			process_gtk_events();

		    sprintf(buf ,"Chat started, type /help for help\n"); 
			append_to_textbox( chat_window, chat_text, buf );
            YahooTextCommand(sess ,"join " ,sess->room); 

			sprintf( buf, "Current Room: %s", sess->room );
			gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, buf );
			process_gtk_events();

            /* set the return value for the program */ 
            sess->rval = 0; 
        } 
        break; 
 
    /* the chat session is ending */ 
    /* socket will be closed after this */ 
    case SHOOK_CHATEND: 
        break; 
 
    /* all sockets closed */ 
    /* end all networking */ 
    case SHOOK_NETEND: 
        break; 
 
    /* program is exiting */ 
    case SHOOK_EXIT: 
		gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
		gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, "Done..." );
		process_gtk_events();
        sprintf(buf ,"Chat Done"); 
        /* set a return value for the program */ 
        r= sess->rval; 
        break; 
    } 
    return r; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* end */ 

/* 
 * copyright David J. Binette 
 * ALL RIGHTS RESERVED 
 */ 
/* ------------------------------------------------------------------------ */ 
/* tab stops are set to 4 */ 
/* ------------------------------------------------------------------------ */ 
/* yahmini is an small yahoo chat client as of July 8 2000 
 * written in portable C for win32 and linux 
 * Its main grace is its small size partially attributable 
 * to consideration of the segment boundarys created in win32 applications. 
 * yahmini has no global volatile variables ,hence 
 * there is only a code segment and a const data segment. 
 * *your* C runtime however may add a whole pile of crap 
 * making it significantly larger. 
 * 
 */ 
 
 
/* ------------------------------------------------------------------------ */ 
 
/* the http request to get cookie ,with username ,password and host */ 
static const char HTTPGet_argU_argP_argH[] = { 
"GET /config/ncclogin?.src=bl&login=%s&passwd=%s&n=1 HTTP/1.0\r\n" 
"Host: %s\r\n" 
"Accept: */*\r\n" 
"User-Agent: Mozilla/4.0\r\n" 
"\r\n" 
}; 
 
/* ------------------------------------------------------------------------ */ 
 
BOOL chatOut  (SESSION *sess ,char *cmd); 
static BOOL yahooPing(SESSION *sess); 
 
/* ------------------------------------------------------------------------ */ 
 
static void showSockError(SESSION *sess ,const char * arg1 ,const char *arg2 
                         ,const char *arg3 ,int errcode) 
{ 
	char buf[4096];

    if (!sess->quit) 
    {
        sprintf(buf 
                ,"%s%s %s failed %s, error %d\n" ,"*** " 
                ,arg1 ,arg2 ,arg3 ,errcode); 
        append_to_textbox( chat_window, chat_text, buf );
    }
} 
 
/* ------------------------------------------------------------------------ */ 
/* 
 * wait for up to 250mS for a packet 
 * RETURNS 
 *      <0 error ,socket is closed 
 *      =0 timeout ,no data available 
 *      >0 data waiting 
 */ 
/* ------------------------------------------------------------------------ */ 
static int sockDataAvailable(SOCKET sock) 
{ 
    int rv; 
    struct timeval  tv; 
    fd_set          fds; 
    tv.tv_sec       = 0; 
    tv.tv_usec      = 100000; 
    FD_ZERO(&fds); 
    FD_SET(sock ,&fds); 
 
    rv = select(sock+1 ,&fds ,NULL ,NULL ,&tv); 
 
    if (rv > 0) 
    {   char x[2]; 
        if (recv(sock ,x ,1 ,MSG_PEEK) < 1) 
            rv = -1; 
    } 
 
    return rv; 
} 
/* ------------------------------------------------------------------------ */ 
 
static int selectRecv(SESSION *sess ,char *buf ,int siz ,int flags) 
{ 
    static int n = 0; 

	while(1)
    { 
		if ( child_pid ) {
			/* we have a child out there somewhere from a /exec command */
			if ( child_timeout < time(NULL) ) {
				/* the child should have finished already, so kill it if */
				/* it's not dead already and then wait for the process so */
				/* we don't leave around zombies */
				kill( child_pid, 9 );
				waitpid( child_pid, NULL, WNOHANG );
				child_pid = 0;
			}
		}

		waitpid( -1, NULL, WNOHANG );

		if ( prof_text )
			display_profile();
/*
		process_gtk_events();
*/

        if (sockDataAvailable(sess->sock)) 
            return recv(sess->sock ,buf ,siz ,flags); 
 
        /* our select sleeps for 250mS ,do a ping approx. every 10 minutes */ 
        if (( ping_minutes ) &&(++n >( 4 * 60 * ping_minutes ))) 
        {   yahooPing(sess); 
            n = 0; 
        } 

		/* set auto-away if needed */
		if (( ! my_status ) &&
			( auto_away ) &&
			( time(NULL) > auto_away_time )) {
			cmd_away( "11" );
		}
    } 

	return FALSE;
} 
 
/* ------------------------------------------------------------------------ */ 
/* General purpose socket read 
 * sock         socket 
 * buf          pointer to data buffer 
 * siz          size of buffer 
 * info         text for error message 
 * 
 * RETURN 
 *        >= 0 number bytes received 
 *        <  0 on error 
 */ 
static int myRecv(SESSION *sess ,char *buf ,int siz ,const char *info) 
{ 
    int cnt = 0; 
    while (siz) 
    {   int b = selectRecv(sess ,buf ,siz ,0); 
        if (b == -1) 
        {
			/*
			showSockError(sess ,info,"incomplete" ,"recv()" ,SockErrNum()); 
			*/
			on_disconnect_activate( NULL, NULL );
            return -1; 
        } 
        if (b == 0) 
            break; 
        cnt += b; 
        buf += b; 
        siz -= b; 
    } 
    return cnt; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* read exactly n bytes 
 * Function will return TRUE if success 
 */ 
 
static BOOL recvN(SESSION *sess ,char *buf ,int cnt ,const char *info) 
{ 
    while (cnt) 
    {   int b = myRecv(sess ,buf ,cnt ,info); 
        if (b < 1) 
        {
			/*
			showSockError(sess ,info ,"incomplete" ,"recv()" ,SockErrNum()); 
			*/
            break; 
        } 
        buf += b; 
        cnt -= b; 
    } 
    return cnt==0; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* send exactly n bytes 
 * Function will return TRUE if success 
 */ 
 
static BOOL sendN(SESSION *sess ,const u_char *buf ,int cnt ,const char *info) 
{ 
    while (cnt) 
    {   int b = send(sess->sock ,(char*)buf ,cnt ,0); 
        if (b < 1) 
        {   showSockError(sess ,info ,"incomplete" ,"send()" ,SockErrNum()); 
            break; 
        } 
        buf += b; 
        cnt -= b; 
    } 
    return cnt==0; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* function to copy from a NULL terminated string 
 * to a NON null terminated string 
 * 
 * Function will return pointer to last byte copied +1 
 */ 
static u_char *appendString(u_char *dst ,const u_char *src) 
{ 
    while (*src) 
        *dst++ = *src++; 
    return dst; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* A function to create a yahoo packet header 
 * 
 * Function will return the next available cell for packet contents 
 */ 
 
static u_char *initYahooHeader(u_char *buf ,u_int code) 
{ 
    memmove(buf ,"YCHT" ,4); 
    *((u_int*)(buf+ 4))=htonl(0x100); 
    *((u_int*)(buf+ 8))=htonl(code); 
/*  *((u_int*)(buf+12))=htonl(length of packet filled inlater);*/ 
    return buf + 16; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* function to send the yahoo packet 
 * the header is updated to contain the accumulated length 
 * 
 * Function will return TRUE if success 
 */ 
BOOL sendYahooPacket(SESSION *sess 
     ,const u_char *bufp ,const u_char *endp ,const char *info) 
{ 
    int n = endp - bufp; 
 
	auto_away_time = time(NULL) + ( auto_away * 60 );

    /* update the packet length count */ 
    *((u_int*)(bufp+12)) = htonl(n-16); 
 
    /* send the packet */ 
    return sendN(sess ,bufp ,n ,info); 
} 
 
/* ------------------------------------------------------------------------ */ 
/* A function to connect to the chat server 
 * Function will return TRUE if success 
 */ 
static BOOL yahooChatConnect(SESSION *sess ,const char *cook) 
{ 
    u_char buf[YAHOOBUFOUTSIZE]; 
    u_char *p = initYahooHeader(buf ,0x01); 
    p    = appendString(p ,(const u_char*)sess->user); 
    *p++ = 0x01; 
    p    = appendString(p ,(const u_char*)cook); 
    return sendYahooPacket(sess ,buf ,p ,"yahooChatConnect"); 
} 
 
/* ------------------------------------------------------------------------ */ 
/* A function that keeps the Yahoo session open. 
 * yahoo will disconnect if this packet is not received within 30 minutes 
 * Function will return TRUE if success 
 */ 
static int yahooPing(SESSION *sess) 
{ 
    u_char  buf[YAHOOBUFOUTSIZE]; 
    u_char *p = initYahooHeader(buf ,0x62); 
    return sendYahooPacket(sess ,buf ,p ,"ping"); 
} 
 
/* ------------------------------------------------------------------------ */ 
 
BOOL YahooTextCommand(SESSION *sess ,const char *cmd ,const char *txt) 
{ 
    u_char buf[YAHOOBUFOUTSIZE]; 
    u_char  *p = initYahooHeader(buf ,0x71); 
    p = appendString(p ,(const u_char*)cmd); 
    p = appendString(p ,(const u_char*)txt); 

	if (( my_status == 11 ) &&
		( ! strstr( cmd, "away " ))) {
		if (( ! strstr( cmd, "tell " )) ||
			( pm_brings_back ))
			cmd_away( "0" );
	} 
 
    return sendYahooPacket(sess ,buf ,p ,*cmd ? cmd : ( *txt ? txt : "send")); 
} 
 
/* ------------------------------------------------------------------------ */ 
/* A function to receive a yahoo chat packet 
 * SESSION *sess = contains a socket connected to the yahoo chat server 
 * 
 * Function will return TRUE if success 
 */ 
 
BOOL yahooRecvData(SESSION *sess) 
{ 
/*  read the yahooheader ,expect 16 bytes 
 *  bytes  0- 3 will always be 59 43 48 54 - or "YCHT" in ASCII. 
 *  bytes  4- 7 are 00 00 01 00 ,perhaps a Yahoo version number. 
 *  bytes  8-11 bytes will be the packet type. 
 *  bytes 12-13 are flag message. generally only needed for errors. 
 *  bytes 14-15 are data length: TCP payload minus 16-byte YCHT header. 
 */ 

 	int result;
    int synccnt = 0; 
    sess->yin.zero = EOS; 

	DBG( 31, "yahooRecvData( %p )\n", sess );
 
    {   char buf[16] ,*p; 
        int  cnt; 
HEADSTART: 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
        p   = buf; 
        cnt = sizeof(buf); 
HEADAGAIN: 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
        if (!recvN(sess ,buf ,cnt ,"Read header")) 
            return FALSE; 
 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
        /* check for a faulty packet */ 
        if (memcmp(buf ,"YCHT\0\0\1\0\0\0\0" ,11)) 
        {   if (!synccnt++) {
				append_to_textbox( chat_window, chat_text, "***Resynch\n" );
            }
            p = buf+1; 
            while (p < (buf+16)) 
            {   if (*p == 'Y') 
                {   cnt = p-buf; 
                    memmove(buf ,p ,16-cnt); 
                    goto HEADAGAIN; 
                } 
                p++; 
            } 
            goto HEADSTART; 
        } 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
        if (synccnt) 
        {
            append_to_textbox( chat_window, chat_text,
            	"***Resynch after a bunch of bytes\n" );
            synccnt = 0; 
        } 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
 
        sess->yin.type = buf[11]; 
        sess->yin.flag = ntohs((u_short)*((u_short*)(buf+12))); 
        sess->yin.size = ntohs((u_short)*((u_short*)(buf+14))); 
 
		DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
        if (sess->yin.size >= YAHOODATAINSIZE) 
        {
			append_to_textbox( chat_window, chat_text,
				"***huge packet rejected\n" );
            goto HEADSTART; 
        } 
    } 
	DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
 
    /* add a null terminator ,just in case */ 
    sess->yin.data[sess->yin.size] = EOS; 
	DBG( 33, "yahooRecvData(), line %d\n", __LINE__ );
 
    /* read the expected amount of data */ 
    result = recvN(sess ,sess->yin.data ,sess->yin.size ,"Read data"); 

	if ( debug_packets )
		dump_packet();

	DBG( 33, "RETURN yahooRecvData() == %d\n", result );
    return result;
} 
 
/* ------------------------------------------------------------------------ */ 
/* A function to resolve and connect to a TCP/IP port 
 * return INVALID_SOCKET on error 
 * otherwise return a valid socket handle 
 */ 
static SOCKET setupConnection(SESSION *sess 
     ,const char *host ,const u_short port ,const int proto ,const char *info) 
{ 
    SOCKET sock; 
    struct sockaddr_in serv_addr; 
	char buf[4096];
 
    sprintf(buf ,"Finding %s..." ,host); 

	gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, buf );
	process_gtk_events();

    memset(&serv_addr ,0 ,sizeof(serv_addr)); 
    serv_addr.sin_addr.s_addr = inet_addr(host); 
 
    if (serv_addr.sin_addr.s_addr == (u_int)INADDR_NONE) 
    {   /* resolve hostname to ipaddr */ 
        struct hostent *hostInfo = gethostbyname(host); 
 
        if (!hostInfo || (hostInfo->h_addrtype != AF_INET)) 
        {   showSockError(sess ,info ,host ,"gethostbyname()" ,SockErrNum()); 
            return INVALID_SOCKET; 
        } 
 
        memset(&serv_addr ,0  ,sizeof(serv_addr)); 
        memmove((void*)&serv_addr.sin_addr.s_addr 
                ,hostInfo->h_addr 
                ,hostInfo->h_length); 
    } 

	gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
	process_gtk_events();
    sprintf(buf ,"Connecting to %s..." ,host); 
	gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, buf );
	process_gtk_events();
 
    serv_addr.sin_family = AF_INET; 
    serv_addr.sin_port   = htons(port); 
 
    sock = socket(AF_INET ,SOCK_STREAM ,proto); 
    if (sock == INVALID_SOCKET) {
        showSockError(sess ,info ,host ,"socket()" ,SockErrNum()); 
	} else if (connect(sock ,(struct sockaddr*)&serv_addr ,sizeof(serv_addr)) == -1) 
    {
		showSockError(sess ,info ,host ,"connect()" ,SockErrNum()); 
        closesocket(sock); 
        sock = INVALID_SOCKET; 
		gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
		process_gtk_events();
    } 

	gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
	process_gtk_events();
 
    return sock; 
} 

/* ------------------------------------------------------------------------ */ 
/* login to yahoo authentication server and get cookie 
 * return TRUE if success 
 */ 
 
static BOOL subLoginToYahoo(SESSION *sess ,char *buf ,int siz) 
{ 
    int   n; 
    char *p; 
    char *hostname = SERVERAUTHHOST; 
 
#if USETINYCRT 
    char *np; 
#define strnext(a ,b) strtoken(a ,b ,&np) 
#else 
#define strnext(a ,b) strtok(a ,b) 
#endif 
 
    sess->cook[0] = EOS; 

	gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, "Login to Yahoo..." );
	process_gtk_events();
 
    sess->sock = setupConnection(sess ,hostname ,SERVERAUTHPORT ,IPPROTO_TCP ,"Login"); 
    if (sess->sock != INVALID_SOCKET) 
    { 
		gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
 
        n = sprintf(buf ,HTTPGet_argU_argP_argH ,sess->user ,sess->pass ,hostname); 
		gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid,
			"Sending Login Sequence..." );
		process_gtk_events();
        n = sendN(sess ,(const u_char*)buf ,n ,"Login"); 
		gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
 
        if (n) {
			gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid,
				"Waiting for Login response..." );
			process_gtk_events();
            n = myRecv(sess ,buf ,siz-1 ,"Login"); 
			gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
		}
 
        closesocket(sess->sock); 
        sess->sock = INVALID_SOCKET; 
 
        if (n < 1) 
        {
			append_to_textbox( chat_window, chat_text, "*** no login data\n" );
		}
        else 
        {   buf[n] = EOS; 
            /* we have to be careful what we output 
             * because some terminals allow keys to be remapped 
             * we dont want to every allow unstripped data to be 
             * sent to the terminal 
             */ 
            for(p=strnext(buf ,"\r\n"); p; p=strnext(NULL ,"\r\n")) 
            { 
                if (!strcmp(p ,"ERROR: Invalid NCC Login")) 
                {   sess->cook[0] = EOS; 
					append_to_textbox( chat_window, chat_text, p );
                    break; 
                } 
 
                if (!strncmp(p ,"Set-Cookie:" ,11)) 
                {   char *q = strchr(p ,';'); 
                    if (q) 
                        *q = EOS; 
                    q = p+11; 
 
                    q = skip_whitespace(q); 
 
                    if (*q) 
                    {   if ((strlen(sess->cook)+strlen(q)+3) >= CHATCOOKSIZE) 
                    	{
                            append_to_textbox( chat_window, chat_text,
                            	"*** Bad cookie\n" );
                        }
                        else 
                        {   if (sess->cook[0]) 
                                strcat(sess->cook ,"; "); 
                            strcat(sess->cook ,q); 
                        } 
                    } 
                } 
            } 
        } 
    } 
	gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
    return sess->cook[0] ? TRUE : FALSE; 
} 
 
/* wrap the call to handle the malloc()/free() 
 * I dont like big stack frames 
 * return TRUE if success 
 */ 
BOOL LoginToYahoo(SESSION *sess) 
{ 
    BOOL r    = FALSE; 
    char *buf = malloc(65534); 
 
    if (buf) 
    {   r    = subLoginToYahoo(sess ,buf ,65533); 
        free(buf); 
    } 
    else 
    {
		append_to_textbox( chat_window, chat_text, "no mem\n" );
    }
    return r; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* login to yahoo chat server and send the login packet 
 * return a TRUE if login ok 
 */ 
BOOL LoginToChat(SESSION *sess) 
{ 
    char tCook[CHATCOOKSIZE+1]; 

	gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid,
		"Login to Chat..." );
	process_gtk_events();

    sess->sock = setupConnection(sess , sess->host ,SERVERCHATPORT ,IPPROTO_TCP ,"Chat"); 
    if (sess->sock != INVALID_SOCKET) 
    {   /* the chat connect function requires a subset of the cookie data 
         * we only want the v= portion so build a temporary cookie 
         */ 
        { char *p = strstr(sess->cook ,"v="); 
            strcpy(tCook ,p?p:sess->cook); 
            if ((p=strstr(tCook ,"; "))!=NULL) 
                *p = EOS; 
        } 
 
        if (!yahooChatConnect(sess ,tCook)) 
        {   closesocket(sess->sock); 
            sess->sock = INVALID_SOCKET; 
        } 
    } 
	gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
	process_gtk_events();
 
    return (sess->sock == INVALID_SOCKET) ? FALSE : TRUE;
} 
 
void subst_args( char *str, char *user )
{
	char tmp[4096];
	char *from, *to;

	DBG( 21, "subst_args( '%s', '%s' )\n", str, user );

	from = str;
	to = tmp;

	if ( ! strchr( str, '%' )) return;

	while( *from ) {
		if (( *from == '%' ) && ( *(from+1) == 'N' )) {
			strcpy( to, YAHOO_COLOR_RED );
			to += strlen( YAHOO_COLOR_RED );
			strcpy( to, user );
			to += strlen( user );
			strcpy( to, YAHOO_COLOR_PURPLE );
			to += strlen( YAHOO_COLOR_PURPLE );
			from += 2;
		} else {
			*(to++) = *(from++);
		}
	}
	*to = '\0';

	strcpy( str, tmp );

	DBG( 22, "RETURN subst_args() == '%s'\n", str );

	return;
}

int check_alias(SESSION *sess, char *emote, char *user )
{
	gchar *listitem[3];
	gchar *emotion;
	gchar *em_everyone;
	gchar *em_user;
	u_char  buf[4096];
	u_char  *k = initYahooHeader(buf ,0x43); 
	u_char  tmp[4096] = "";
	u_char  tmp2[4096] = "";
	u_char  tmp_em[21];
	int i, x;
	int found = -1;

	DBG( 21, "check_alias( %p, '%s', '%s' )\n", sess, emote, user );

	listitem[0] = emotion;
	listitem[1] = em_everyone;
	listitem[2] = em_user;

	k = appendString(k , "emote\001" );

	for( i = 0; i < strlen( emote ); i++ ) {
		emote[i] = tolower( emote[i] );
	}

	i = 0;
	while(1) {
		if ( !gtk_clist_get_text(GTK_CLIST(chat_emotions), i, 0, &emotion ))
			break;

		strcpy( tmp_em, emotion );
		for( x = 0; x < strlen( tmp_em ); x++ ) {
			tmp_em[x] = tolower( tmp_em[x] );
		}

		if ( ! strcmp( emote, tmp_em )) {
			emotion = tmp_em;
			found = i;
			break;
		}

		i++;
	}

	if ( found >= 0 ) {
		if ( user[0] ) {
			gtk_clist_get_text(GTK_CLIST(chat_emotions), found, 2,
				&em_user );

			strcpy( tmp2, em_user );

			if ( strchr( tmp2, '$' ))
				strcpy( tmp2, replace_args( tmp2, user ));

			em_user = tmp2;

			if (( em_user[0] == '/' ) ||
				( em_user[0] == '!' )) {
				strcpy( tmp, em_user );
				chatOut( sess, tmp );
				DBG( 22, "RETURN check_alias() == TRUE\n" );

				return TRUE;
			}

			strcpy( tmp, YAHOO_COLOR_PURPLE );
			strcat( tmp, em_user );

		} else {
			gtk_clist_get_text(GTK_CLIST(chat_emotions), found, 1,
				&em_everyone );

			if (( em_everyone[0] == '/' ) ||
				( em_everyone[0] == '!' )) {
				chatOut( sess, em_everyone );
				DBG( 22, "RETURN check_alias() == TRUE\n" );

				return TRUE;
			}

			strcat( tmp, YAHOO_COLOR_PURPLE );
			strcpy( tmp, em_everyone );
		}

		subst_escs( tmp );

		if ( my_status ) {
			cmd_away( "0" );
		} 

		k = appendString(k , tmp );
		sendYahooPacket(sess ,buf ,k , "send"); 

		DBG( 22, "RETURN check_alias() == TRUE\n" );

		return( TRUE );
	} else {
		DBG( 22, "RETURN check_alias() == FALSE\n" );

		return( FALSE );
	}
}

/* ------------------------------------------------------------------------ */ 
/* send a chat command 
 * Note: it modifies the string in place 
 * return FALSE if we should quit 
 */ 
BOOL chatOut(SESSION *sess ,char *cmd) 
{ 
	BOOL r    = TRUE; 
	char *pre = ""; 
	char *txt = ""; 
	char *args;
	char buf[1025] = "";
	regex_t smiley_regex;

	DBG( 21, "chatOut( %p, '%s' )\n", sess, cmd );

	cmd = skip_whitespace( cmd );

	auto_away_time = time(NULL) + ( auto_away * 60 );

	switch(cmd[0]) { 
		case EOS: 
			return( TRUE );
			break; 
		case ':':
			/* differentiate between smileys and emotes */
			regcomp( &smiley_regex, "^:[)({}|<>DP]", REG_NOSUB );
			
			if ( ! regexec( &smiley_regex, cmd, 0, NULL, 0 )) {
				/* it's a smiley */
				pre = "say say\001"; 
				txt = cmd;
				subst_escs( txt );
			} else {
				/* it's not */
				sprintf( buf, "/emote %s", cmd+1 );
				try_command( buf );
				return( TRUE );
			}
			break;
		case '!':
			sprintf( buf, "/exec %s", cmd+1 );
			try_command( buf );
			return( TRUE );
			break;

		/* for testing */
#if 0
		case '^':
			{
				GdkPixmap *st_pixmap = NULL;
				GdkBitmap *st_mask = NULL;
				GtkStyle *user_style = NULL;

				user_style = gtk_widget_get_style( chat_text );

				st_pixmap = gdk_pixmap_create_from_xpm_d(
					chat_text->window, &st_mask,
					&user_style->bg[GTK_STATE_NORMAL],
					(gchar **)pixmap_status_ignore );

				gdk_draw_pixmap( chat_text->window,
					chat_text->style->fg_gc[GTK_STATE_NORMAL],
					st_pixmap,
					100, 100, 100, 100, 20, 20 );
					/*
					event->area.x, event->area.y,
					event->area.x, event->area.y,
                    event->area.width, event->area.height);
					*/
			}
			break;
#endif

		case '/': 
			if ( try_command( cmd ))
				return( TRUE );

			args = find_whitespace( cmd + 1);
			if ( *args ) {
				*args = '\0';
				args++;
				args = skip_whitespace( args );
			}

			if ( check_alias( sess, cmd + 1, args ))
				return( TRUE );

			/* if we get here we dunno what it is so send straight as is */
			pre = cmd;
        	break; 
 
		default: 
			pre = "say say\001"; 
			if (( cmd[0] == '\\' ) &&
				(( cmd[1] == '/' ) ||
				 ( cmd[1] == ':' ) ||
				 ( cmd[1] == '!' ))) {
				txt = cmd +1;
			} else {
				txt = cmd; 
			}

			subst_escs( txt );
			break;
    } 

	if ( use_color ) {
		if ( ! strcmp( use_color, "black" )) {
			strcat( buf, YAHOO_COLOR_BLACK );
		} else if ( ! strcmp( use_color, "blue" )) {
			strcat( buf, YAHOO_COLOR_BLUE );
		} else if ( ! strcmp( use_color, "cyan" )) {
			strcat( buf, YAHOO_COLOR_CYAN );
		} else if ( ! strcmp( use_color, "gray" )) {
			strcat( buf, YAHOO_COLOR_GRAY );
		} else if ( ! strcmp( use_color, "green" )) {
			strcat( buf, YAHOO_COLOR_GREEN );
		} else if ( ! strcmp( use_color, "pink" )) {
			strcat( buf, YAHOO_COLOR_PINK );
		} else if ( ! strcmp( use_color, "purple" )) {
			strcat( buf, YAHOO_COLOR_PURPLE );
		} else if ( ! strcmp( use_color, "yellow" )) {
			strcat( buf, "\033[#ffff00m" );
		} else if ( ! strcmp( use_color, "orange" )) {
			strcat( buf, "\033[#ffc800m" );
		} else if ( ! strcmp( use_color, "red" )) {
			strcat( buf, YAHOO_COLOR_RED );
		}
	}

	if ( use_bold )
		strcat( buf, YAHOO_STYLE_BOLDON );

	if ( use_italics )
		strcat( buf, YAHOO_STYLE_ITALICON );

	if ( use_underline )
		strcat( buf, YAHOO_STYLE_UNDERLINEON );

	if ( filter_command ) {
		strcat( buf, filter_text( txt ));
	} else {
		strcat( buf, txt );
	}

    if (*pre || *buf ) 
        r = YahooTextCommand( sess, pre, buf ); 
 
	DBG( 22, "RETURN chatOut() == %d\n", r );

    return r; 
} 
 
/* ------------------------------------------------------------------------ */ 
/* the mask controls which fields are assigned to the 
 *  four pointers passed on the invocation to parse(). 
 * Not all of the pointers to parse() are populated. 
 *  that is controlled by the special chars in mask. 
 * args is only used as a late test to validate that 
 *  the right number of assignments has occured. 
 */ 
typedef struct YPARSE_DEF { 
/* fields controlling how the incoming packet is scanned */ 
    const short         type;      /* packet type */ 
    const short         args;      /* # of data fields */ 
    const char  * const mask;      /* variable mask */ 
/* fields controlling how the packet is displayed */ 
    const char  * const show;      /* a format string for fprintf */ 
} YPARSE; 
 
/* ------------------------------------------------------------------------ */ 
/* 
 * this routines scans the sess->yin.data and populates a series of pointers. 
 * the fmt string tokens direct us to scan for and optionally stores 
 * pointers to data.  The data in sess->yin->dta is modified and terminated 
 * with ASCII NULL. The variable args are pointers that will receive the 
 * a pointer to the scanned text withing the yahoo packet. 
 * 
 * we honour the following special characters 
 * $ expect text followed by \300\200 ,store the pointer 
 * * expect text followed by \300\200 ,do not assign it 
 * 
 * returns the number of fields scanned and assigned 
 * 
 * assumes the data is null terminated 
 */ 
 
static short __cdecl parse(SESSION *sess ,const YPARSE *yp ,char **endp ,...) 
{ 
    short       assigned = 0; 
    char        *pdata   = sess->yin.data; 
    const char  *limit   = sess->yin.data + sess->yin.size; 
    const char  *fmt     = yp->mask; 
    va_list     argptr; 
 
    va_start(argptr ,endp); 
 
    while (*fmt) 
    { 
        if (*fmt=='$') /* assign a string or number  */ 
        {   assigned++; 
            *va_arg( argptr ,char**) = pdata; 
        } 
 
        while (pdata < limit)   /* skip to the next field */ 
        {   if ((*pdata == '\300') && (*(pdata+1) == '\200')) 
            {   *pdata = EOS; 
                pdata+=2; 
                break; 
            } 
            pdata++; 
        } 
        fmt++; 
    } 
    va_end(argptr); 
 
    *endp = pdata;  /* store the position of the next data*/ 
    return assigned; 
} 
 
/* ------------------------------------------------------------------------ */ 
 
static void showErrUnknown(SESSION *sess ,const char *msg) 
{ 
	char buf[4096];

    sprintf(buf 
            ,"%s type=0x%02x ,flag=0x%04x ,size=%u\n%s\n" 
            ,msg 
            ,sess->yin.type 
            ,sess->yin.flag 
            ,sess->yin.size 
            ,sess->yin.data 
            ); 
	append_to_textbox( chat_window, chat_text, buf );
} 
 
/* ------------------------------------------------------------------------ */ 
 
static char * show(char *fielddata[] ,char c ) 
{ 
    char *p = ""; 
 
    if ((c>='0') && (c<='3'))   p = fielddata[c-'0']; 
    else if (1) switch(c) { 
                    case '+' :  p = ATTR_BOLD   ; break; 
                    case '-' :  p = ATTR_RESET  ; break; 
                    case 'b' :  p = YAHOO_COLOR_BLUE   ; break; 
                    case 'c' :  p = YAHOO_COLOR_CYAN   ; break; 
                    case 'g' :  p = ATTR_HGREEN ; break; 
                    case 'G' :  p = ATTR_LGREEN ; break; 
                    case 'p' :  p = YAHOO_COLOR_PURPLE ; break; 
                    case 'r' :  p = YAHOO_COLOR_RED    ; break; 
                    case 'y' :  p = ATTR_YELLOW ; break; 
                    } 
    return p; 
} 
 
/* ------------------------------------------------------------------------ */ 
#if 0 
line wrap makes it difficult to read on 80 col terminals 
so the packet number ,type and fields of interest are shown here 
    0x01 "login"   name,-,-,-,-,mail 
    0x02 "logout"  text 
    0x11 "enters"  room,text,spam,list 
    0x12 "leaves"  room,name 
    0x17 "invite"  text,room,name 
    0x21 "move"    name,room,away,text 
    0x25 "notuser" text 
    0x41 "says"    -room,name,text 
    0x42 "thinks"  -room,name,text 
    0x43 "emotes"  -room,name,text 
    0x44 "spam"    text 
    0x45 "private" name,user,text 
    0x46 "away"    text 
    0x47 "dice"    name,text 
    0x64 "help"    text 
    0x68 "buddy"   mode 
    0x69 "mail"    mail 
    0x70 "yahoo"   text 
    0x71 "err71"   text 
    0x80 "grafiti" codes 
    0xff "mutant"  ??? 
#endif 

/* ------------------------------------------------------------------------ */ 
#define FIELD(n) show(f ,yp->show[n] ) 
/* note that our table does not include packet 0x80 ,we ignore it */ 
void showYahooPacket(SESSION *sess) 
{ 
    int r; 
    char *endp = sess->yin.data; 
    char *f[4] = { "" ,"" ,"" ,""}; /* receives pointers to packet fields */ 
    static const YPARSE yparse[] = 
 
/*    {{0x01 ,2 ,"$****$" ,"r0-+1- Welcome %s%s%s, you have %s%s%s email\n"} */
    {{0x01 ,2 ,"$****$" ,"r0-+1- Welcome %s%s%s\n"                       } 
    ,{0x02 ,1 ,"$"      ,"0      %s\n"                                   } 
    ,{0x11 ,4 ,"$$$*$"  ,"b3g-012%s%s %senters%s the room\n"             } 
    ,{0x12 ,2 ,"$$"     ,"c1G-0  %s%s %sleaves%s the room\n"             } 
    ,{0x17 ,3 ,"$$$"    ,"r2-y1- %s%s%s invites you to %s%s%s\n"         } 
    ,{0x21 ,4 ,"$$$$"   ,"0123   %s %s(%s) '%s'\n"                       } 
    ,{0x25 ,1 ,"$"      ,"0      unknown user %s\n"                      } 
    ,{0x41 ,2 ,"*$$"    ,"r0--1- %s%s%s: %s%s%s\n"                       } 
    ,{0x42 ,2 ,"*$$"    ,"r0-p1- %s%s%s: %s%s%s\n"                       } 
    ,{0x43 ,2 ,"*$$"    ,"r0-p1- %s%s%s: %s%s%s\n"                       } 
    ,{0x44 ,1 ,"$"      ,"0      spam > %s\n"                            } 
    ,{0x45 ,3 ,"$$$"    ,"b0-1+2-%s%s%s <private to %s> %s%s%s\n"        } 
    ,{0x46 ,1 ,"$"      ,"+b-0   %s%sBuddy: %s%s%s%s\n"                  } 
    ,{0x47 ,2 ,"$$"     ,"r0-1   %s%s%s <dice> %s\n"                     } 
    ,{0x64 ,1 ,"$"      ,"0      help %s\n"                              } 
    ,{0x68 ,2 ,"$$"     ,"1      buddy %s\n"                             } 
    ,{0x69 ,1 ,"$"      ,"+0-    You have %s email(s).\n"                } 
    ,{0x70 ,1 ,"$"      ,"0      yahoo %s\n"                             } 
    ,{0x71 ,1 ,"$"      ,"0      err71 %s\n"                             } 
    ,{0xff ,0 ,""       ,"       "                                       } 
    /* pkt #arg mask     order   outputText                             */ 
    }; 
    const YPARSE *yp = yparse; 

	char buf[4096];
	char msg[512];
	char *ptr = NULL;
	char *user = NULL;
	char ruser[51] = "";
	gchar *listitem[2];
	static char last_packet[1025] = "";
	int row;
	int iggy;
	char *lf;
	int lf_count = 0;
	int auto_ignore_this_msg = 0;
	GtkWidget *tmp_widget = NULL;
	GtkStyle *user_style = NULL;
	time_t time_now;
	struct tm *tm_now;
	PM_SESSION *pm_sess;
	GList *pm_lptr;


#ifdef SHOW_STATUS_PIXMAPS
	GdkPixmap *st_pixmap = NULL;
	GdkBitmap *st_mask = NULL;
#else
	char status[5] = "";
#endif

	DBG( 21, "showYahooPacket( %p )\n", sess );

    /* we ignore a graffiti packet */ 
    if (( sess->yin.type == 0x44 ) ||
		( sess->yin.type == 0x80 ))
        return; 
 
    if (sess->yin.flag < 0) { showErrUnknown(sess ,"error"); return;  } 
 
    /* scan the parse table ,stopping at the found item ,or the last entry */ 
    while((yp->type != 0xff) && (yp->type != sess->yin.type)) 
        yp++; 
 
    if (yp->type == 0xff)   { showErrUnknown(sess ,"unknown"); return; } 

    /* assign pointers according to the format mask */ 
    r = parse(sess ,yp ,&endp ,&f[0] ,&f[1] ,&f[2] ,&f[3]); 
 
    if (r != yp->args)      /* not all variables populated */ 
    { 
        sprintf( buf
                ,"bad packet (%d/%d args) type=%x ,flag=0x%04x ,size=%u\n%s\n" 
                ,r 
                ,yp->args 
                ,sess->yin.type 
                ,sess->yin.flag 
                ,sess->yin.size 
                ,sess->yin.data
                ); 
		append_to_textbox( chat_window, chat_text, buf );

		sprintf( buf, "arg0 = '%s'\narg1 = '%s'\narg2 = '%s'\narg3 = '%s'\n",
			f[0], f[1], f[2], f[3] );
		append_to_textbox( chat_window, chat_text, buf );

		if ( debug_packets )
			dump_packet();
 
        return; 
    } 
 
	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	switch( sess->yin.type ) {
		case 0x01:
		case 0x02:
		case 0x21:
		case 0x25:
		case 0x41:
		case 0x42:
		case 0x43:
		case 0x45:
		case 0x47:
					strcpy( ruser, f[0] );
					break;
		case 0x12:
					strcpy( ruser, f[1] );
					break;
		case 0x17:
					strcpy( ruser, f[2] );
					break;
		case 0x11:
					strcpy( ruser, f[3] );
					break;
	}

    /* special handling for a room enter */ 
    if (sess->yin.type == 0x11) 
    { 
    	if ( strstr( f[0], "Danger Will" )) {
			return;
    	}

		tmp_widget = lookup_widget( chat_window, "users" );
		user_style = gtk_widget_get_style( tmp_widget );

        if ( strchr(f[3] ,',') || strcmp(f[0] ,sess->room )) 
        {   strncpy(sess->room ,f[0] ,CHATROOMSIZE); 
            sess->room[CHATROOMSIZE] = EOS; 

			gtk_statusbar_pop( GTK_STATUSBAR(chat_status), st_cid );
			sprintf( buf, "Current Room: %s", f[0] );
			gtk_statusbar_push( GTK_STATUSBAR(chat_status), st_cid, buf );
			process_gtk_events();

            sprintf( buf
                ,"%sYou are now in %s%s\n%s%s%s\nYou see here %s\n" 
                ,show(f ,'b' )   /* room name in yellow  */ 
                ,show(f ,'0' ) 
                ,show(f ,'-' ) 
                ,show(f ,'c' )   /* room message in bold */ 
                ,show(f ,'1' ) 
                ,show(f ,'-' ) 
                ,f[3]);                     /* list of guests       */ 
			append_to_textbox( chat_window, chat_text, buf );

			/* clear & fill in the user list box */
			gtk_clist_freeze(GTK_CLIST(chat_users));
			gtk_clist_clear(GTK_CLIST(chat_users));

			user = f[3];
			ptr = strchr( user, ',' );
			while( ptr ) {
				*ptr = '\0';
				listitem[0] = user;
				listitem[1] = user;
#ifdef CASE_INSENSITIVE_USER_LIST
				ptr = user;
				while( *ptr ) {
					*ptr = tolower( *ptr );
					ptr++;
				}
#endif

#ifdef SHOW_STATUS_PIXMAPS
				if ( ignore_check( user )) {
					st_pixmap = gdk_pixmap_create_from_xpm_d(
						tmp_widget->window, &st_mask,
						&user_style->bg[GTK_STATE_NORMAL],
						(gchar **)pixmap_status_ignore );

				} else {
					if ( mute_check( user )) {
						st_pixmap = gdk_pixmap_create_from_xpm_d(
							tmp_widget->window, &st_mask,
							&user_style->bg[GTK_STATE_NORMAL],
							(gchar **)pixmap_status_mute );
					} else {
						st_pixmap = gdk_pixmap_create_from_xpm_d(
							tmp_widget->window, &st_mask,
							&user_style->bg[GTK_STATE_NORMAL],
							(gchar **)pixmap_status_here );
					}
				}
#else
				if ( ignore_check( user )) {
					strcpy( status, "IG" );
				} else {
					if ( mute_check( user )) {
						strcpy( status, "MU" );
					} else {
						strcpy( status, "" );
					}
				}
#endif
			
				row = find_user_row( user );
				if ( row < 0 ) {
					row = gtk_clist_insert(GTK_CLIST(chat_users), 0, listitem );
					gtk_clist_set_text(GTK_CLIST(chat_users), row, 1, user );
#ifdef SHOW_STATUS_PIXMAPS
					gtk_clist_set_pixmap(GTK_CLIST(chat_users), row, 0,
						st_pixmap, st_mask );
					gdk_pixmap_unref( st_pixmap );
					gdk_bitmap_unref( st_mask );
#else
					gtk_clist_set_text(GTK_CLIST(chat_users), row, 0, status );
#endif
				}

				user = ptr + 1;
				ptr = strchr( user, ',' );
			}
			listitem[0] = user;
			listitem[1] = user;
#ifdef CASE_INSENSITIVE_USER_LIST
			ptr = user;
			while( *ptr ) {
				*ptr = tolower( *ptr );
				ptr++;
			}
#endif

#ifdef SHOW_STATUS_PIXMAPS
			if ( ignore_check( user )) {
				st_pixmap = gdk_pixmap_create_from_xpm_d(
					tmp_widget->window, &st_mask,
					&user_style->bg[GTK_STATE_NORMAL],
					(gchar **)pixmap_status_ignore );
			} else {
				if ( mute_check( user )) {
					st_pixmap = gdk_pixmap_create_from_xpm_d(
						tmp_widget->window, &st_mask,
						&user_style->bg[GTK_STATE_NORMAL],
						(gchar **)pixmap_status_mute );
				} else {
					st_pixmap = gdk_pixmap_create_from_xpm_d(
						tmp_widget->window, &st_mask,
						&user_style->bg[GTK_STATE_NORMAL],
						(gchar **)pixmap_status_here );
				}
			}
#else
			if ( ignore_check( user )) {
				strcpy( status, "IG" );
			} else {
				if ( mute_check( user )) {
					strcpy( status, "MU" );
				} else {
					strcpy( status, "" );
				}
			}
#endif
				
			row = gtk_clist_insert(GTK_CLIST(chat_users), 0, listitem );
			gtk_clist_set_text(GTK_CLIST(chat_users), row, 1, user );
#ifdef SHOW_STATUS_PIXMAPS
			gtk_clist_set_pixmap(GTK_CLIST(chat_users), row, 0,
				st_pixmap, st_mask );
			gdk_pixmap_unref( st_pixmap );
			gdk_bitmap_unref( st_mask );
#else
			gtk_clist_set_text(GTK_CLIST(chat_users), row, 0, status );
#endif
			gtk_clist_sort(GTK_CLIST(chat_users));
			gtk_clist_thaw(GTK_CLIST(chat_users));

            return; 
        } 
    } /* end of 0x111 packet */

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	if (( sess->yin.type == 0x17 ) &&
		( sess->yin.flag == 0x00 )) {
/*
		( f[1][0] == '\0' )) {
*/
		/* inviting is 0 = "user has been invited to this room", 1 & 2 blank */
		/* invited  is 0 = ??, 1 == room, 2 == user */
		strcpy( buf, YAHOO_COLOR_RED );
		strcat( buf, f[0] );
		ptr = strchr( buf, ' ' );
		*ptr = '\0';
		strcat( buf, YAHOO_COLOR_BLACK );
		strcat( buf, " has been invited to this room.\n" );
		append_to_textbox( chat_window, chat_text, buf );
		return;
	}

	/* update users table when users enter/leave */
	if ( sess->yin.type == 0x11 ) {
		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
		tmp_widget = lookup_widget( chat_window, "users" );
		user_style = gtk_widget_get_style( tmp_widget );

		/* entering */
		listitem[0] = f[3];
		listitem[1] = f[3];
#ifdef CASE_INSENSITIVE_USER_LIST
		ptr = f[3];
		while( *ptr ) {
			*ptr = tolower( *ptr );
			ptr++;
		}
#endif

		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

#ifdef SHOW_STATUS_PIXMAPS
		if (( iggy = ignore_check( f[3] )) != 0 ) {
			st_pixmap = gdk_pixmap_create_from_xpm_d(
				tmp_widget->window, &st_mask,
				&user_style->bg[GTK_STATE_NORMAL],
				(gchar **)pixmap_status_ignore );
		} else {
			if (( iggy = mute_check( f[3] )) != 0 ) {
				st_pixmap = gdk_pixmap_create_from_xpm_d(
					tmp_widget->window, &st_mask,
					&user_style->bg[GTK_STATE_NORMAL],
					(gchar **)pixmap_status_mute );
			} else {
				st_pixmap = gdk_pixmap_create_from_xpm_d(
					tmp_widget->window, &st_mask,
					&user_style->bg[GTK_STATE_NORMAL],
					(gchar **)pixmap_status_here );
			}
		}
#else
		if (( iggy = ignore_check( f[3] )) != 0 ) {
			DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
			strcpy( status, "IG" );
		} else {
			DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
			if (( iggy = mute_check( f[3] )) != 0 ) {
				DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
				strcpy( status, "MU" );
			} else {
				DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
				strcpy( status, "" );
			}
		}
#endif

		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

		gtk_clist_freeze(GTK_CLIST(chat_users));
		row = gtk_clist_append(GTK_CLIST(chat_users), listitem );
		gtk_clist_set_text(GTK_CLIST(chat_users), row, 1, f[3] );
#ifdef SHOW_STATUS_PIXMAPS
		gtk_clist_set_pixmap(GTK_CLIST(chat_users), row, 0,
			st_pixmap, st_mask );
		gdk_pixmap_unref( st_pixmap );
		gdk_bitmap_unref( st_mask );
#else
		gtk_clist_set_text(GTK_CLIST(chat_users), row, 0, status );
#endif
		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

		gtk_clist_sort(GTK_CLIST(chat_users));
		gtk_clist_thaw(GTK_CLIST(chat_users));

		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

		if ( iggy )
			return;

		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
	} else if ( sess->yin.type == 0x12 ) {
		DBG( 201, "showYahooPacket() line %d\n", __LINE__ );
		tmp_widget = lookup_widget( chat_window, "users" );
		user_style = gtk_widget_get_style( tmp_widget );

		/* leaving */
		listitem[0] = buf;

		r = find_user_row( f[1] );

		if ( r >= 0 ) gtk_clist_remove( GTK_CLIST(chat_users), r );

		if (( follow_user ) &&
			( ! strcmp( f[1], follow_user ))) {
			sprintf( buf, "/goto %s", follow_user );
			try_command( buf );
		}

		if ( ignore_check( f[1] )) {
			DBG( 22, "RETURN showYahooPacket(), ignoring user '%s'\n", f[1] );
			return;
		}

		if ( mute_check( f[1] )) {
			DBG( 22, "RETURN showYahooPacket(), muting user '%s'\n", f[1] );
			return;
		}
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

    /* maybe we dont show enter/leave messages */ 
    if (( ! show_enters ) && 
        (( sess->yin.type == 0x11 ) ||
		 ( sess->yin.type == 0x12 ) ||
		 ( sess->yin.type == 0x68 ))) {
        return; 
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	/* check ignore list */
	if ( ruser[0] ) {
		if ( ignore_check( ruser )) {
			DBG( 22, "RETURN showYahooPacket(), ignoring user '%s', packet "
				"type %x\n", ruser, sess->yin.type );
			return;
		}

		if ( mute_check( ruser )) {
			DBG( 22, "RETURN showYahooPacket(), muting user '%s', packet "
				"type %x\n", ruser, sess->yin.type );
			return;
		}
	}

#if THIS_WAS_THE_OLD_IGNORE_CHECK_CODE
	if ((( sess->yin.type == 0x41 ) || /* say */
		 ( sess->yin.type == 0x42 ) || /* think */
		 ( sess->yin.type == 0x43 ) || /* emote */
		 ( sess->yin.type == 0x45 ) || /* private */
		 ( sess->yin.type == 0x21 )) &&  /* status */
		(( ignore_check( f[0] )) ||
		 ( mute_check( f[0] ))))
		return;
#endif

	
 
	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	if ( sess->yin.type == 0x46 ) {
		tmp_widget = lookup_widget( chat_window, "users" );
		user_style = gtk_widget_get_style( tmp_widget );

		listitem[0] = buf;

		/* ignore yahoo spam disquised as away packet */
		/* usernames can't be longer than 31 I believe, and yahoo sometimes */
		/* sends spam using type 0x46 with username length of about 150 */
		if ( strlen( f[0] ) > 64 )
			return;

		r = find_user_row( f[0] );

		DBG( 201, "showYahooPacket() line %d, user '%s' == %d\n",
			__LINE__, f[0], r );

		if ( r >= 0 )
		{
#ifdef SHOW_STATUS_PIXMAPS
			st_pixmap = gdk_pixmap_create_from_xpm_d(
				tmp_widget->window, &st_mask,
				&user_style->bg[GTK_STATE_NORMAL],
				(gchar **)pixmap_status_away );
			gtk_clist_set_pixmap(GTK_CLIST(chat_users), r, 0,
				st_pixmap, st_mask );
#else
			gtk_clist_set_text(GTK_CLIST(chat_users), r, 0, "AW" );
#endif
		}

		/* should we ignore status changes */
		if ( ! show_statuses )
			return;

		/* return explicitly now because yahoo is using this to spam */
		/*
		return;
		*/
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	if ( sess->yin.type == 0x21 ) {
		tmp_widget = lookup_widget( chat_window, "users" );
		user_style = gtk_widget_get_style( tmp_widget );

		iggy = ignore_check( f[0] );

		if ( ! iggy )
			iggy = mute_check( f[0] );

#ifdef SHOW_STATUS_PIXMAPS
		st_pixmap = gdk_pixmap_create_from_xpm_d(
			tmp_widget->window, &st_mask,
			&user_style->bg[GTK_STATE_NORMAL],
			(gchar **)pixmap_status_away );
#else
		strcpy( status, "AW" );
#endif
		sprintf( buf, "%s%s%s is %saway%s (",
			YAHOO_COLOR_BLUE, f[0], YAHOO_COLOR_BLACK,
			YAHOO_STYLE_BOLDON, YAHOO_STYLE_BOLDOFF );

		if ( f[2][0] == 0x31 && f[2][1] == 0x30 ) {
			strcat( buf, f[3] );
			strcat( buf, ")\n" );
		} else if ( f[2][0] == 0x31 && f[2][1] == 0x31 ) {
			strcat( buf, "Auto-Away)\n" );
#ifdef SHOW_STATUS_PIXMAPS
			st_pixmap = gdk_pixmap_create_from_xpm_d(
				tmp_widget->window, &st_mask,
				&user_style->bg[GTK_STATE_NORMAL],
				(gchar **)pixmap_status_idle );
#else
			strcpy( status, "AA" );
#endif
		} else if ( f[2][0] == 0x31 && f[2][1] == 0x32 ) {
			strcat( buf, "Invisible)\n" );
		} else if ( f[2][0] == 0x30 ) {
#ifdef SHOW_STATUS_PIXMAPS
			if ( iggy ) {
				if ( ignore_check( f[0] )) {
					st_pixmap = gdk_pixmap_create_from_xpm_d(
						tmp_widget->window, &st_mask,
						&user_style->bg[GTK_STATE_NORMAL],
						(gchar **)pixmap_status_ignore );
				} else {
					st_pixmap = gdk_pixmap_create_from_xpm_d(
						tmp_widget->window, &st_mask,
						&user_style->bg[GTK_STATE_NORMAL],
						(gchar **)pixmap_status_mute );
				}
			} else {
				st_pixmap = gdk_pixmap_create_from_xpm_d(
					tmp_widget->window, &st_mask,
					&user_style->bg[GTK_STATE_NORMAL],
					(gchar **)pixmap_status_here );
			}
#else
			if ( iggy ) {
				if ( ignore_check( f[0] )) {
					strcpy( status, "IG" );
				} else {
					strcpy( status, "MU" );
				}
			} else {
				strcpy( status, "" );
			}
#endif
			sprintf( buf, "%s%s%s is now %sback%s\n",
				YAHOO_COLOR_BLUE, f[0], YAHOO_COLOR_BLACK,
				YAHOO_STYLE_BOLDON, YAHOO_STYLE_BOLDOFF );
		} else if (( f[2][0] >= 0x31 ) && ( f[2][0] <= 0x39 )) {
			strcat( buf, yahoo_status[f[2][0] - 0x30] );
			strcat( buf, ")\n" );
		}

		row = find_user_row( f[0] );
#ifdef SHOW_STATUS_PIXMAPS
		gtk_clist_set_pixmap(GTK_CLIST(chat_users), row, 0,
			st_pixmap, st_mask );
		gdk_pixmap_unref( st_pixmap );
		gdk_bitmap_unref( st_mask );
#else
		gtk_clist_set_text(GTK_CLIST(chat_users), row, 0, status );
#endif

		if ( iggy ) {
			return;
		}

		/* should we ignore status changes */
		if ( ! show_statuses )
			return;
	} else {
    	sprintf( buf
           ,yp->show+7 
       	   ,FIELD(0),FIELD(1),FIELD(2),FIELD(3),FIELD(4),FIELD(5),FIELD(6)); 
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	/* check the message against the auto ignore regex list */
	if (((( sess->yin.type == 0x41 ) ||
		  ( sess->yin.type == 0x42 ) ||
		  ( sess->yin.type == 0x43 )) &&
		 ( auto_ignore_check( f[1] ))) ||
		(( sess->yin.type == 0x45 ) &&
		 ( auto_ignore_check( f[2] )))) {
		if ( f[0][0] )
			auto_ignore_this_msg = 1;
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	/* auto-reply if necessary */
	if (( sess->yin.type == 0x45 ) &&
		( ! auto_ignore_this_msg ) &&
		( my_status ) &&
		( auto_reply_when_away )) {
		if ( auto_reply_msg ) {
			sprintf( msg, "/tell %s %s", f[0], auto_reply_msg );
			chatOut( sess, msg );
		}
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	/* User says/thinks/emotes */
	if ( sess->yin.type == 0x11 ) {
		sprintf( buf, "%s%s %senters%s the room\n",
			YAHOO_COLOR_BLUE, f[3], YAHOO_COLOR_GREEN, YAHOO_COLOR_BLACK );
		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored ))
			append_to_textbox_color( chat_window, chat_text, buf );
	} else if ( sess->yin.type == 0x12 ) {
		sprintf( buf, "%s%s %sleaves%s the room\n",
			YAHOO_COLOR_BLUE, f[1], YAHOO_COLOR_GREEN, YAHOO_COLOR_BLACK );
		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored ))
			append_to_textbox_color( chat_window, chat_text, buf );
	} else if ( sess->yin.type == 0x17 ) {
		sprintf( buf, "%sYou have been invited to (%s%s%s%s%s) by %s%s%s%s%s\n",
			YAHOO_COLOR_PURPLE, YAHOO_COLOR_RED, YAHOO_STYLE_BOLDON,
			f[1], YAHOO_STYLE_BOLDOFF, YAHOO_COLOR_PURPLE,
			YAHOO_COLOR_BLUE, YAHOO_STYLE_BOLDON, f[2], YAHOO_STYLE_BOLDOFF,
			YAHOO_COLOR_BLACK );
		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored ))
			append_to_textbox_color( chat_window, chat_text, buf );
	} else if (( sess->yin.type == 0x41 ) ||
		( sess->yin.type == 0x42 ) ||
		( sess->yin.type == 0x43 )) {
		if ( suppress_mult ) {
			sprintf( buf, "%s: %s", f[0], f[1] );
			if ( ! strcmp( buf, last_packet ))
				return;

			strcpy( last_packet, buf );
		}

		/* check for lots of linefeeds */
		if ( limit_lfs ) {
			lf = strchr( f[1], '\n' );
			while( lf ) {
				lf_count++;
				lf = strchr( lf + 1, '\n' );
			}
			if ( lf_count > limit_lfs ) {
				lf = strchr( f[1], '\n' );
				while( lf ) {
					*lf = '|';
					lf = strchr( lf + 1, '\n' );
				}
			}
		}

		if ( chat_timestamp ) {
			time_now = time( NULL );
			tm_now = localtime( &time_now );
			sprintf( buf, "%s[%d/%d/%d %02d:%02d] ",
				YAHOO_COLOR_BLACK,
				tm_now->tm_mon + 1, tm_now->tm_mday, tm_now->tm_year + 1900,
				tm_now->tm_hour, tm_now->tm_min );
			append_to_textbox_color( chat_window, chat_text, buf );
		}

		if ( strcmp( f[0], sess->user )) {
			sprintf( buf, "%s%s%s%s%s",
				YAHOO_STYLE_BOLDON, YAHOO_COLOR_RED, f[0],
				YAHOO_COLOR_BLACK, YAHOO_STYLE_BOLDOFF );
		} else {
			sprintf( buf, "%s%s%s%s%s",
				YAHOO_STYLE_BOLDON, YAHOO_COLOR_BLUE, f[0],
				YAHOO_COLOR_BLACK, YAHOO_STYLE_BOLDOFF );
		}

		if ( sess->yin.type == 0x41 )
			strcat( buf, ": " );
		else
			strcat( buf, " " );

		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored ))
			append_to_textbox_color( chat_window, chat_text, buf );

		if ( sess->yin.type == 0x41 ) {
			sprintf( buf, "%s%s%s\n",
				YAHOO_COLOR_BLACK, f[1], YAHOO_COLOR_BLACK );
			if (( ! auto_ignore_this_msg ) ||
				( disp_auto_ignored ))
				append_to_textbox( chat_window, chat_text, buf );
		} else {
			sprintf( buf, "%s%s%s\n",
				YAHOO_COLOR_PURPLE, f[1], YAHOO_COLOR_BLACK );
			if (( ! auto_ignore_this_msg ) ||
				( disp_auto_ignored ))
				append_to_textbox_color( chat_window, chat_text, buf );
		}
	} else if ( sess->yin.type == 0x45 ) {
		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored )) {

			if (( ! pm_in_sep_windows ) ||
				( auto_ignore_this_msg )) {
				if ( chat_timestamp ) {
					time_now = time( NULL );
					tm_now = localtime( &time_now );
					sprintf( buf, "%s[%d/%d/%d %02d:%02d] ",
						YAHOO_COLOR_BLACK,
						tm_now->tm_mon + 1, tm_now->tm_mday,
						tm_now->tm_year + 1900,
						tm_now->tm_hour, tm_now->tm_min );
					append_to_textbox_color( chat_window, chat_text, buf );
				}

				sprintf( buf, "%s%s%s%s%s <private to %s%s%s%s%s> %s%s%s\n",
					YAHOO_COLOR_RED, YAHOO_STYLE_BOLDON, f[0],
					YAHOO_COLOR_BLACK, YAHOO_STYLE_BOLDOFF,
					YAHOO_COLOR_BLUE, YAHOO_STYLE_BOLDON, f[1],
					YAHOO_COLOR_BLACK, YAHOO_STYLE_BOLDOFF,
					YAHOO_STYLE_BOLDON, f[2], YAHOO_STYLE_BOLDOFF );

				append_to_textbox_color( chat_window, chat_text, buf );
			} else {
				/* here we would send to a private pm session */
				/* duplicate to both for now for testing */
				if (( pm_lptr = find_pm_session( f[0] )) != NULL ) {
					pm_sess = pm_lptr->data;
				} else {
					/* open a new window if one doesn't exist already */
					pm_sess = (PM_SESSION *)malloc( sizeof( PM_SESSION ));
					pm_sess->pm_window = create_pm_session();
					pm_sess->pm_text = lookup_widget( pm_sess->pm_window,
						"pms_text" );
					pm_sess->pm_user = strdup( f[0] );

					sprintf( buf, "Private Chat: %s\n", f[0] );
					gtk_window_set_title( GTK_WINDOW(pm_sess->pm_window), buf );

					pm_list = g_list_append( pm_list, pm_sess );
				}
				gtk_widget_show( pm_sess->pm_window );

				if ( auto_raise_pm ) {
					gdk_window_raise( pm_sess->pm_window->window );
				}

				if ( chat_timestamp_pm ) {
					time_now = time( NULL );
					tm_now = localtime( &time_now );
					sprintf( buf, "%s[%d/%d/%d %02d:%02d] ",
						YAHOO_COLOR_BLACK,
						tm_now->tm_mon + 1, tm_now->tm_mday,
						tm_now->tm_year + 1900,
						tm_now->tm_hour, tm_now->tm_min );
					append_to_textbox_color( pm_sess->pm_window,
						pm_sess->pm_text, buf );
				}

				sprintf( buf, "%s%s%s%s%s: %s\n",
					YAHOO_COLOR_RED, YAHOO_STYLE_BOLDON, f[0],
					YAHOO_COLOR_BLACK, YAHOO_STYLE_BOLDOFF, f[2] );
				append_to_textbox_color( pm_sess->pm_window, pm_sess->pm_text,
					buf );
			}
		}
	} else if ( sess->yin.type == 0x68 ) {
		sprintf( buf, "%sBuddy:%s ", YAHOO_COLOR_BLUE, YAHOO_COLOR_BLACK );
		if ( strlen( f[0] ) == 1 ) {
			if ( strlen( f[1] ) == 0 )	
				return;

			/* 0 - log off */
			/* 1 - log on */
			/* 3 - list of friends currently online */
			strcat( buf, YAHOO_COLOR_RED );
			strcat( buf, f[1] );
			strcat( buf, YAHOO_COLOR_BLACK );
			switch ( f[0][0] ) {
				case '0':	remove_online_friend( f[1] );
							strcat( buf, " has logged OFF.\n" );
							append_to_textbox_color( chat_window, chat_text,
								buf );
							if (( pm_lptr = find_pm_session( f[1] )) != NULL ) {
								pm_sess = pm_lptr->data;
								append_to_textbox_color( pm_sess->pm_window,
									pm_sess->pm_text, buf );
							}
							break;
				case '1':	add_online_friend( f[1] );
							strcat( buf, " has logged ON.\n" );
							append_to_textbox_color( chat_window, chat_text,
								buf );
							if (( pm_lptr = find_pm_session( f[1] )) != NULL ) {
								pm_sess = pm_lptr->data;
								append_to_textbox_color( pm_sess->pm_window,
									pm_sess->pm_text, buf );
							}
							break;
				case '3':	build_online_friends_list();
							show_friends();
							break;
			}
		} else {
			strcpy( buf, f[1] );
			strcat( buf, "\n" );
			append_to_textbox_color( chat_window, chat_text, buf );
		}
	} else {
		if (( ! auto_ignore_this_msg ) ||
			( disp_auto_ignored ))
			append_to_textbox( chat_window, chat_text, buf );
	}

	DBG( 201, "showYahooPacket() line %d\n", __LINE__ );

	if ( auto_ignore_this_msg ) {
		if ( f[0][0] ) {
			if ( ! ignore_check( f[0] ))
				ignore_toggle( f[0] );

			sprintf( buf, "Gyach - auto-ignored %s%s%s\n",
				YAHOO_COLOR_RED, f[0], YAHOO_COLOR_BLACK );
			append_to_textbox( chat_window, chat_text, buf );
		}
	}
} 
#undef FIELD 
 
/* ------------------------------------------------------------------------ */ 

int build_toolbar( GtkWidget *chat_window ) {
	GtkWidget *tgl_bold;
	GtkWidget *tgl_italics;
	GtkWidget *tgl_underline;
	GtkWidget *chat_toolbar;
	GdkPixmap *st_pixmap = NULL;
	GdkBitmap *st_mask = NULL;
	GtkWidget *tmp_pixmap;
	GtkStyle *user_style = NULL;
	int tsize = 20;        /* size of toolbar buttons in pixels wide & high */

	chat_toolbar = lookup_widget( chat_window, "chat_toolbar" );

	user_style = gtk_widget_get_style( chat_toolbar );

	st_pixmap = gdk_pixmap_create_from_xpm_d(
		chat_toolbar->window, &st_mask,
		&user_style->bg[GTK_STATE_NORMAL],
		(gchar **)pixmap_bold );

	tmp_pixmap = gtk_pixmap_new( st_pixmap, st_mask );

	tgl_bold = gtk_toolbar_append_element (GTK_TOOLBAR (chat_toolbar),
		GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, " B ", "Send your text in Bold",
		NULL, tmp_pixmap, NULL, NULL);
	gtk_widget_ref (tgl_bold);
	gtk_object_set_data_full (GTK_OBJECT (chat_window), "tgl_bold", tgl_bold,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tgl_bold);
	gtk_widget_set_usize (tgl_bold, tsize, tsize);

	st_pixmap = gdk_pixmap_create_from_xpm_d(
		chat_toolbar->window, &st_mask,
		&user_style->bg[GTK_STATE_NORMAL],
		(gchar **)pixmap_italic );

	tmp_pixmap = gtk_pixmap_new( st_pixmap, st_mask );

	tgl_italics = gtk_toolbar_append_element (GTK_TOOLBAR (chat_toolbar),
		GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, " I ", "Send your text in italic",
		NULL, tmp_pixmap, NULL, NULL);
	gtk_widget_ref (tgl_italics);
	gtk_object_set_data_full (GTK_OBJECT (chat_window), "tgl_italics",
		tgl_italics, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tgl_italics);
	gtk_widget_set_usize (tgl_italics, tsize, tsize);

	st_pixmap = gdk_pixmap_create_from_xpm_d(
		chat_toolbar->window, &st_mask,
		&user_style->bg[GTK_STATE_NORMAL],
		(gchar **)pixmap_underline );

	tmp_pixmap = gtk_pixmap_new( st_pixmap, st_mask );

	tgl_underline = gtk_toolbar_append_element (GTK_TOOLBAR (chat_toolbar),
		GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, " U ",
		"Send your text in underline", NULL, tmp_pixmap, NULL, NULL);
	gtk_widget_ref (tgl_underline);
	gtk_object_set_data_full (GTK_OBJECT (chat_window), "tgl_underline",
		tgl_underline, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (tgl_underline);
	gtk_widget_set_usize (tgl_underline, tsize, tsize);

	gtk_signal_connect (GTK_OBJECT (tgl_bold), "toggled",
		GTK_SIGNAL_FUNC (on_tgl_bold_toggled), NULL);
	gtk_signal_connect (GTK_OBJECT (tgl_italics), "toggled",
		GTK_SIGNAL_FUNC (on_tgl_italics_toggled), NULL);
	gtk_signal_connect (GTK_OBJECT (tgl_underline), "toggled",
		GTK_SIGNAL_FUNC (on_tgl_underline_toggled), NULL);

	return( 0 );
}

GtkWidget *build_chat_window( void ) {
	GtkWidget *chat_window;
	GtkWidget *tmp_widget;

	chat_window = create_window1();
	gtk_window_set_title( GTK_WINDOW(chat_window), "Gyach v" VERSION );

	/* set default window sizes from saved values */
	if ( remember_position )
		gtk_widget_set_uposition( chat_window, chat_window_x, chat_window_y );
	gtk_widget_set_usize( chat_window, chat_window_width, chat_window_height );
	gtk_window_set_default_size( GTK_WINDOW(chat_window), chat_window_width,
		chat_window_height );

	tmp_widget = lookup_widget( chat_window, "chat_text" );
	gtk_widget_set_usize( tmp_widget, chat_text_width, -2 );
	tmp_widget = lookup_widget( chat_window, "scrolledwindow3" );
	gtk_widget_set_usize( tmp_widget, -2, chat_user_height );

	gtk_widget_show( chat_window );

	build_toolbar( chat_window );

	st_cid = gtk_statusbar_get_context_id( GTK_STATUSBAR(chat_status),
		"Gyach" );

	set_menu_connected( 0 );

	/* set any options that need the window created */
	set_config();

	chat_user_menu = create_user_menu();
	emote_menu = create_emote_menu();
	setup_menu = create_setup_menu();

	return( chat_window );
}


GtkWidget *build_login_window( void ) {
	GtkWidget *tmp_widget;
	GList *serverlist = NULL;

	login_window = create_login_window();

	if ( ! password )
		password = strdup( "" );

	tmp_widget = lookup_widget( login_window, "chat_username" );

	if ( login_list )
		gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), login_list );

	tmp_widget = lookup_widget( login_window, "chat_password" );
	gtk_entry_set_text( GTK_ENTRY(tmp_widget), password );

	if ( ! favroom_list ) {
		favroom_list = g_list_append( favroom_list,
			"Linux, FreeBSD, Solaris" );
		favroom_list = g_list_append( favroom_list,
			"Linux, FreeBSD, Solaris:1" );
		favroom_list = g_list_append( favroom_list, "Programming" );
		favroom_list = g_list_append( favroom_list, "Programming:1" );
		favroom_list = g_list_append( favroom_list, "Lobby" );
		favroom_list = g_list_append( favroom_list, "Lobby:1" );
	}

	tmp_widget = lookup_widget( login_window, "chat_room" );

	gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), favroom_list );

	/* make cs5 the default for now by putting it first */
	serverlist = g_list_append( serverlist, "cs5.chat.yahoo.com" );

	serverlist = g_list_append( serverlist, "cs.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs2.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs3.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs4.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs6.chat.yahoo.com" );

	tmp_widget = lookup_widget( login_window, "chat_server" );
	gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), serverlist );

	if ( login_list ) {
		strcpy( sess->user, (char*)login_list->data );
	} else {
		strcpy( sess->user, "" );
	}
	strcpy( sess->pass, password );
	strcpy( sess->room, (char*)favroom_list->data );
	strcpy( sess->host, (char*)serverlist->data );

	return( login_window );
}

int __cdecl main( int argc, char **argv ) {
	GtkWidget *chat_window;
	char gtkrc_filename[1024];
	struct stat gtkrc_stat;

	DBG( 1, "Gyach v%s\n", VERSION );
	DBG( 1, "%s\n", GYACH_URL );
	DBG( 1, "Debug Level set to: %d\n", DBG_LEVEL );

	memset(sess ,0 ,sizeof(SESSION)); 
	sess->rval = 1; 
	sess->sock = INVALID_SOCKET; 

	GlobalSessionHook = sess; 

	gtk_set_locale ();

	gtk_init (0,0);

	sprintf( gtkrc_filename, "%s/.gyach/gtkrc", getenv( "HOME" ));
	if ( ! stat( gtkrc_filename, &gtkrc_stat )) {
		gtk_rc_parse( gtkrc_filename );
	}

	gyach_init();

	chat_window = build_chat_window();
	gtk_widget_show( chat_window );

	login_window = build_login_window();

	if ( ! auto_login ) {
		gtk_widget_show( login_window );
	} else {
		login_to_yahoo_chat();
	}

	gdk_threads_enter();

	gtk_main();

	return( 0 );
}

int __cdecl orig_main(int argc ,const char **argv) 
{ 
	GtkWidget *window1;
	GtkWidget *tmp_widget;
	GList *serverlist = NULL;
	char gtkrc_filename[1024];
	struct stat gtkrc_stat;

	DBG( 1, "Gyach v%s\n", VERSION );
	DBG( 1, "%s\n", GYACH_URL );
	DBG( 1, "Debug Level set to: %d\n", DBG_LEVEL );

	memset(sess ,0 ,sizeof(SESSION)); 
	sess->rval = 1; 
	sess->sock = INVALID_SOCKET; 

	gtk_set_locale ();

	gtk_init (0,0);

	sprintf( gtkrc_filename, "%s/.gyach/gtkrc", getenv( "HOME" ));
	if ( ! stat( gtkrc_filename, &gtkrc_stat )) {
		gtk_rc_parse( gtkrc_filename );
	}

	gyach_init();

	if ( argc != 4 ) {
	login_window = create_login_window();

	if ( ! password )
		password = strdup( "" );

	tmp_widget = lookup_widget( login_window, "chat_password" );
	gtk_entry_set_text( GTK_ENTRY(tmp_widget), password );

	tmp_widget = lookup_widget( login_window, "chat_username" );
	gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(tmp_widget)->entry), username );
	gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), login_list );

	if ( ! favroom_list ) {
		favroom_list = g_list_append( favroom_list,
			"Linux, FreeBSD, Solaris" );
		favroom_list = g_list_append( favroom_list,
			"Linux, FreeBSD, Solaris:1" );
		favroom_list = g_list_append( favroom_list, "Programming" );
		favroom_list = g_list_append( favroom_list, "Programming:1" );
		favroom_list = g_list_append( favroom_list, "Lobby" );
		favroom_list = g_list_append( favroom_list, "Lobby:1" );
	}

	tmp_widget = lookup_widget( login_window, "chat_room" );
	if ( def_room ) {
		strcpy( roomname, def_room );
		gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(tmp_widget)->entry),
			roomname );
	}

	gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), favroom_list );

	/* make cs4 the default for now by putting it first */
	serverlist = g_list_append( serverlist, "cs5.chat.yahoo.com" );

	serverlist = g_list_append( serverlist, "cs.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs2.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs3.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs4.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs6.chat.yahoo.com" );

	/* yahoo only has 6 servers it seems, thought there were more....
	serverlist = g_list_append( serverlist, "cs7.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs8.chat.yahoo.com" );
	serverlist = g_list_append( serverlist, "cs9.chat.yahoo.com" );
	*/

	tmp_widget = lookup_widget( login_window, "chat_server" );
	gtk_combo_set_popdown_strings( GTK_COMBO(tmp_widget), serverlist );

	strcpy( username, "testing" );

	if ( ! auto_login ) {
		gtk_widget_show(login_window);

		while( ! valid_user ) {
			process_gtk_events();
			usleep( 10000 );
		}

		/* user canceled login screen */
		if ( ! username[0] ) {
			exit( 0 );
		}
	}

	tmp_widget = lookup_widget( login_window, "chat_username" );
	strcpy( sess->user,
		gtk_entry_get_text( GTK_ENTRY(GTK_COMBO(tmp_widget)->entry)));

	tmp_widget = lookup_widget( login_window, "chat_password" );
	strcpy( sess->pass, gtk_entry_get_text( GTK_ENTRY(tmp_widget)));

	if ( password ) {
		free( password );
		password = NULL;
	}

	/* remember password, will be saved when we write_config() */
	if ( remember_password ) {
		password = strdup( gtk_entry_get_text( GTK_ENTRY(tmp_widget)));
	}

	tmp_widget = lookup_widget( login_window, "chat_room" );
	strcpy( sess->room,
		gtk_entry_get_text( GTK_ENTRY(GTK_COMBO(tmp_widget)->entry)));

	tmp_widget = lookup_widget( login_window, "chat_server" );
	strcpy( sess->host,
		gtk_entry_get_text( GTK_ENTRY(GTK_COMBO(tmp_widget)->entry)));

	gtk_widget_destroy(login_window);

	} else {
		strcpy( sess->user, argv[1] );
		strcpy( sess->pass, argv[2] );
		if ( remember_password ) {
			password = strdup( argv[2] );
		}
		strcpy( sess->room, argv[3] );
		strcpy( sess->host, "cs5.chat.yahoo.com" );
	}

	window1 = create_window1();
	gtk_window_set_title( GTK_WINDOW(window1), "Gyach v" VERSION );

	/* set default window sizes from saved values */
	if ( remember_position )
		gtk_widget_set_uposition( window1, chat_window_x, chat_window_y );
	gtk_widget_set_usize( window1, chat_window_width, chat_window_height );
	gtk_window_set_default_size( GTK_WINDOW(window1), chat_window_width,
		chat_window_height );

	tmp_widget = lookup_widget( window1, "chat_text" );
	gtk_widget_set_usize( tmp_widget, chat_text_width, -2 );
	tmp_widget = lookup_widget( window1, "scrolledwindow3" );
	gtk_widget_set_usize( tmp_widget, -2, chat_user_height );

	gtk_widget_show(window1);

	st_cid = gtk_statusbar_get_context_id( GTK_STATUSBAR(chat_status),
		"Gyach" );

	/* set any options that need the window created */
	set_config();

	chat_user_menu = create_user_menu();
	emote_menu = create_emote_menu();
	setup_menu = create_setup_menu();

	process_gtk_events();

	gdk_threads_enter();

	if (SessionHook(sess ,SHOOK_PROG ,0 ,NULL))          /* set io handels */ 
	{
     if              (SessionHook(sess ,SHOOK_NET ,0 ,NULL)) 
     { if            (LoginToYahoo(sess)) 
       { if          (SessionHook(sess ,SHOOK_CHAT ,0 ,NULL)) 
         { if        (LoginToChat(sess)) 
           { if      (SessionHook(sess ,SHOOK_RECV ,0 ,NULL)) 
             {
			   while (yahooRecvData(sess)) 
               {
				process_gtk_events();
                 if  (SessionHook(sess ,SHOOK_DATA ,0 ,NULL)) 
                 {    showYahooPacket(sess); 
                 } 
               } 
                     SessionHook(sess ,SHOOK_CHATEND ,0 ,NULL); 
                     closesocket(sess->sock); 
                     sess->sock = INVALID_SOCKET; 
                     sess->quit = 0; 
             } 
           } 
         } 
       } 
                     SessionHook(sess ,SHOOK_NETEND ,0 ,NULL); 
     } 
 } 
	gdk_threads_leave();
 return              SessionHook(sess ,SHOOK_EXIT ,0 ,NULL); 
} 
 
/* ------------------------------------------------------------------------ */ 
/* end */ 
